from __future__ import absolute_import

from briscas.models import Suite, Colors

from six.moves import input, range


def ask_for_input(prompt, allowed, input_fn=input, exit_fn=exit):
    i = str(input_fn(prompt))
    while i not in allowed and i != 'exit':
        i = str(input_fn(prompt))
    if i == 'exit':
        exit_fn()
    return i


def is_better(a, b, life):
    if a.suite == life.suite and b.suite != life.suite:
        return True
    elif a.suite != life.suite and b.suite == life.suite:
        return False
    elif a.suite == b.suite:
        return (
            a.number == 1 or
            (a.number == 3 and b.number != 1) or
            (b.number not in [1, 3] and a.number > b.number)
        )
    else:  # different non-life suites. first card wins.
        return True


def b(i, j):
    """Block coordinates"""
    return [(i, j), (i, j+1), (i+1, j), (i+1, j+1)]


CARD_LENGTH = L = 14
CARD_HEIGHT = H = 10
SYMBOL_LENGTH = SL = 2
SYMBOL = {
    Suite.ORO: Colors.YELLOW + '#' + Colors.RESET,
    Suite.BASTON: Colors.GREEN + '#' + Colors.RESET,
    Suite.ESPADA: Colors.BLUE + '#' + Colors.RESET,
    Suite.COPA: Colors.RED + '#' + Colors.RESET,
}
TOP_BOTTOM_BORDER = [' '] + ['-'] * (L - 2) + [' ']
EMPTY_LINE = ['|'] + [' '] * (L - 2) + ['|']
L2 = int(L / 2) - 1  # HALF LENGTH
H2 = int(H / 2) - 1  # HALF HEIGHT
L4 = int(L2 / 2)
H4 = int(H2 / 2)
SYMBOL_LOCATIONS = {
    1: [b(L2, H2)],
    2: [b(L2, H4), b(L2, H2 + H4)],
    3: [b(L2, H4 - 1), b(L2, H2), b(L2, H2 + H4 + 1)],
    4: [b(L4, H4), b(L4, H2 + H4),
        b(L2 + L4, H4), b(L2 + L4, H2 + H4)],
    5: [b(L4, H4), b(L4, H2 + H4),
        b(L2 + L4, H4), b(L2 + L4, H2 + H4),
        b(L2, H2)],
    6: [b(L4, H4 - 1), b(L4, H2), b(L4, H2 + H4 + 1),
        b(L2 + L4, H4 - 1), b(L2 + L4, H2), b(L2 + L4, H2 + H4 + 1)],
    7: [b(L4, H4 - 1), b(L4, H2), b(L4, H2 + H4 + 1),
        b(L2 + L4, H4 - 1), b(L2 + L4, H2), b(L2 + L4, H2 + H4 + 1),
        b(L2, H4)],
    8: [b(L4, H4 - 1), b(L4, H2), b(L4, H2 + H4 + 1),
        b(L2 + L4, H4 - 1), b(L2 + L4, H2), b(L2 + L4, H2 + H4 + 1),
        b(L2, H4), b(L2, H2 + H4)],
    9: [b(L4, H4 - 1), b(L4, H2), b(L4, H2 + H4 + 1),
        b(L2 + L4, H4 - 1), b(L2 + L4, H2), b(L2 + L4, H2 + H4 + 1),
        b(L2, H4 - 1), b(L2, H2), b(L2, H2 + H4 + 1)],
    10: [(L2 - 2, H2 - 2), (L2 - 2, H2 - 1),
         (L2 - 2, H2), (L2 - 2, H2 + 1), (L2 - 2, H2 + 2),
         (L2 + 1, H2 - 2), (L2 + 1, H2 - 1), (L2 + 1, H2),
         (L2 + 1, H2 + 1), (L2 + 1, H2 + 2),
         (L2 + 2, H2 - 2), (L2 + 2, H2 + 2),
         (L2 + 3, H2 - 2), (L2 + 3, H2 - 1), (L2 + 3, H2),
         (L2 + 3, H2 + 1), (L2 + 3, H2 + 2)],
    11: [(L2 - 2, H2 - 2), (L2 - 2, H2 - 1), (L2 - 2, H2),
         (L2 - 2, H2 + 1), (L2 - 2, H2 + 2),
         (L2 + 3, H2 - 2), (L2 + 3, H2 - 1), (L2 + 3, H2),
         (L2 + 3, H2 + 1), (L2 + 3, H2 + 2)],
    12: [(L2 - 2, H2 - 2), (L2 - 2, H2 - 1), (L2 - 2, H2),
         (L2 - 2, H2 + 1), (L2 - 2, H2 + 2),
         (L2 + 1, H2 - 2), (L2 + 1, H2), (L2 + 1, H2 + 1),
         (L2 + 1, H2 + 2),
         (L2 + 2, H2 - 2), (L2 + 2, H2), (L2 + 2, H2 + 2),
         (L2 + 3, H2 - 2), (L2 + 3, H2 - 1), (L2 + 3, H2), (L2 + 3, H2 + 2)],
}
# Flatten blocks.
for i in range(1, 10):
    SYMBOL_LOCATIONS[i] = [t for block in SYMBOL_LOCATIONS[i] for t in block]


def hand_string(hand):
    # Create empty matrix
    matrix = []
    matrix.append(TOP_BOTTOM_BORDER * len(hand))
    for i in range(H - 2):
        matrix.append(EMPTY_LINE * len(hand))
    matrix.append(TOP_BOTTOM_BORDER * len(hand))

    # Insert symbols
    o = 0  # offset
    for card in hand:
        s = SYMBOL[card.suite]
        matrix[1][o + L - 2] = str(card.number)[-1]
        matrix[H - 2][o + 1] = str(card.number)[0]
        if len(str(card.number)) > 1:  # Then add other digit
            matrix[1][o + L - 3] = str(card.number)[0]
            matrix[H - 2][o + 2] = str(card.number)[1]
        for (i, j) in SYMBOL_LOCATIONS[card.number]:
            matrix[j][o + i] = s
        o += CARD_LENGTH

    # Build into string
    string = '\n'.join([''.join(line) for line in matrix])
    return string



from __future__ import print_function
from __future__ import absolute_import

from briscas.game import Game
from briscas.players import HumanPlayer, RandomPlayer, LocalPlayer
from briscas.util import ask_for_input

from six.moves import input, range


def choose_player(name, input_fn=input):
    i = ask_for_input('Choose %s >>> ' % name,
                      [str(i) for i in range(5)],
                      input_fn=input_fn)
    return {
        '0': lambda: HumanPlayer(name),
        '1': lambda: RandomPlayer(name),
        '2': lambda: LocalPlayer(name),
    }[i]()


def main(print_fn=print, input_fn=input):
    print_fn('===== Players')
    print_fn('0. Human Player')
    print_fn('1. Random Player')
    print_fn('2. Local Player')
    p1 = choose_player('P1', input_fn=input_fn)
    p2 = choose_player('P2', input_fn=input_fn)

    game = Game(p1, p2, verbose=True)
    game.play()



from setuptools import setup

setup(
    name='briscas',
    packages=['briscas'],
    version='1.1',
    description='Library to model the briscas card game.',
    author='Bryan Collazo',
    author_email='bcollazo2010@gmail.com',
    license='MIT',
    url='https://github.com/bcollazo/briscas',
    download_url='https://github.com/bcollazo/briscas/archive/v1.1.zip',
    keywords=['briscas', 'card', 'game', 'terminal', 'naipes'],
    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: MIT License',
        'Topic :: Games/Entertainment',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    install_requires=[],
    entry_points={
        'console_scripts': ['briscas=briscas.main:main']
    }
)



from __future__ import print_function
from __future__ import absolute_import
import random

from briscas.util import is_better, ask_for_input, hand_string

from six.moves import range

NUMBER_ORDERING = [1, 3] + list(range(12, 3, -1)) + [2]


class Player(object):
    def __init__(self, name):
        self.name = name
        self.pile = []

    def init(self, hand):
        self.hand = hand

    def push_to_pile(self, cards):
        self.pile.extend(cards)

    def take_into_hand(self, card):
        self.hand.append(card)

    def has_cards(self):
        return len(self.hand) > 0

    # TODO: Future to include piles
    def play(self, life_card, thrown=None):
        raise Exception('Not yet implemented!')  # pragma: no cover


class HumanPlayer(Player):
    def __init__(self, name, print_fn=print):
        super(HumanPlayer, self).__init__(name)
        self.print_fn = print_fn

    def play(self, life_card, thrown=None):
        self.print_fn(hand_string(self.hand))
        playable = [str(i + 1) for i in range(len(self.hand))]
        prompt = 'Choose (%s) >>> ' % (', '.join([i for i in playable]))
        i = ask_for_input(prompt, playable)
        return self.hand.pop(int(i) - 1)


class RandomPlayer(Player):
    def play(self, life_card, thrown=None):
        i = random.randint(0, len(self.hand) - 1)
        return self.hand.pop(i)


class LocalPlayer(Player):
    def least_good_index(self, cards, life_card):
        min_index = None
        min_rank = None
        for i, card in enumerate(cards):
            card_rank = 0
            if card.suite == life_card.suite:
                card_rank += 36
            card_rank += (12 - NUMBER_ORDERING.index(card.number))

            if min_rank is None or min_rank > card_rank:
                min_index = i
                min_rank = card_rank
        return min_index

    def play(self, life_card, thrown=None):
        if thrown is None:
            i = self.least_good_index(self.hand, life_card)
            return self.hand.pop(i)

        betters = []
        for i, card in enumerate(self.hand):
            if is_better(card, thrown, life_card):
                betters.append(card)
        if len(betters) == 0:
            i = self.least_good_index(self.hand, life_card)
            return self.hand.pop(i)
        j = self.least_good_index(betters, life_card)
        i = self.hand.index(betters[j])
        return self.hand.pop(i)



from __future__ import print_function
from __future__ import absolute_import
import json

from briscas.models import Deck, POINTS
from briscas.players import HumanPlayer
from briscas.util import is_better, hand_string

from six.moves import range


class Game:

    @staticmethod
    def score(pile):
        return sum([c.points() for c in pile])

    def __init__(self, player1, player2, verbose=False, print_fn=print):
        self.verbose = verbose
        self.deck = Deck()
        self.player1 = player1
        self.player2 = player2
        self.winner = None
        self.plays = []  # list of maps
        self.life_card = self.deck.peek_last_card()  # for recording
        self.print_fn = print_fn

    def _deal(self):
        cards = [self.deck.pop() for i in range(6)]
        self.player1.init([cards[0], cards[2], cards[4]])
        self.player2.init([cards[1], cards[3], cards[5]])

    def _print(self, message):
        if self.verbose:
            self.print_fn(message)

    def log_play(self, commander, c_play, follower, f_play):
        play = {
            commander.name: {
                'hand': [c.to_dict() for c in commander.hand],
                'play': c_play.to_dict()
            },
            follower.name: {
                'hand': [c.to_dict() for c in follower.hand],
                'play': f_play.to_dict()
            },
            'commander': commander.name
        }
        self.plays.append(play)

    def play(self):
        self._deal()

        commander = self.player1
        follower = self.player2
        life_card = self.deck.peek_last_card()
        while self.player1.has_cards() or self.player2.has_cards():
            self._print('\033[2J')
            self._print('%s LIFE CARD: %s %s' % (
                '=' * 11, str(life_card), '=' * 11))
            c_play = commander.play(life_card)
            if not isinstance(commander, HumanPlayer):
                self._print(hand_string([c_play]))
                self._print('\n\n' + '=' * 30)
            f_play = follower.play(life_card, thrown=c_play)
            if not isinstance(commander, HumanPlayer):
                self._print(hand_string([c_play]))
                self._print('\n\n' + '=' * 30)
            self.log_play(commander, c_play, follower, f_play)

            if is_better(c_play, f_play, life_card):
                self._print('%s takes it.' % (commander.name))
                commander.push_to_pile([c_play, f_play])
            else:
                self._print('%s takes it.' % (follower.name))
                follower.push_to_pile([c_play, f_play])
                commander, follower = (follower, commander)

            if self.deck.has_cards():
                commander.take_into_hand(self.deck.pop())
                follower.take_into_hand(self.deck.pop())

        p1_score = Game.score(self.player1.pile)
        p2_score = Game.score(self.player2.pile)
        self._print('%s points: %d [%s]' % (
            self.player1.name, p1_score,
            [c for c in self.player1.pile if c.number in POINTS]))
        self._print('%s points: %d [%s]' % (
            self.player2.name, p2_score,
            [c for c in self.player2.pile if c.number in POINTS]))

        self.set_winner(p1_score, p2_score)
        return self.winner

    def set_winner(self, p1_score, p2_score):
        if p1_score > p2_score:
            self._print('===== %s WINS =====' % (self.player1.name))
            self.winner = self.player1
        elif p1_score == p2_score:
            self._print("Empate!")
        else:
            self._print('===== %s WINS =====' % (self.player2.name))
            self.winner = self.player2

    def to_json(self):
        data = {
            'winner': None if self.winner is None else self.winner.name,
            'plays': self.plays,
            'life_suite': self.life_card.suite.name
        }
        return json.dumps(data)



from __future__ import absolute_import
import enum
import random

from six.moves import range

POINTS = {1: 11, 3: 10, 12: 4, 11: 3, 10: 2}


class Colors:
    RESET = '\033[0m'
    BOLD = '\033[01m'
    RED = '\033[31m'
    GREEN = '\033[32m'
    BLUE = '\033[34m'
    YELLOW = '\033[93m'


class Suite(enum.Enum):
    ORO = 1
    COPA = 2
    ESPADA = 3
    BASTON = 4

    def __str__(self):
        if self == Suite.ORO:
            return Colors.YELLOW + self.name + Colors.RESET
        elif self == Suite.COPA:
            return Colors.RED + self.name + Colors.RESET
        elif self == Suite.ESPADA:
            return Colors.BLUE + self.name + Colors.RESET
        elif self == Suite.BASTON:
            return Colors.GREEN + self.name + Colors.RESET


class Card:
    def __init__(self, number, suite):
        if (suite not in Suite or
                not isinstance(number, int) or
                number < 1 or number > 13):
            raise Exception('Invalid card')
        self.number = number
        self.suite = suite

    def points(self):
        if self.number in POINTS:
            return POINTS[self.number]
        else:
            return 0

    def to_dict(self):
        return {
            'number': self.number,
            'suite': self.suite.name,
        }

    def __repr__(self):
        if self.number in [1, 3]:
            number = Colors.BOLD + str(self.number) + Colors.RESET
        else:
            number = self.number
        return "[%s %s]" % (number, self.suite)


class Deck:
    def __init__(self):
        self._cards = []
        for s in Suite:
            for n in range(1, 13):
                self._cards.append(Card(n, s))
        random.shuffle(self._cards)

    def pop(self):
        return self._cards.pop(0)

    def peek_last_card(self):
        return self._cards[-1]

    def has_cards(self):
        return len(self._cards) > 0
